def f(fixed, col_sep, cols, row, row_sep, padding):
    s = col_sep
    for col in cols:
        symbol = fixed.get((row, col))
        if symbol:
            s += ' '*padding + symbol
        else:
            s += ' '*padding + '.'   
        s += col_sep
    return s

def dict_to_string(fixed, rows, cols, padding = 0, row_sep = '', col_sep = '', top = '', bottom = ''):
    """
    Returns a puzzle string of dimension 'boxsize' from a dictionary of 
    'fixed' cells.
    
    padding : number of spaces between adjacent symbols
    row_end : a string added to the last symbol of every row 
    col_sep : a string added to the last symbol of every column
    
    """
    s = top
    for row in rows[:-1]:
        s += f(fixed, col_sep, cols, row, row_sep, padding)
        s += row_sep
    s += f(fixed, col_sep, cols, rows[-1], row_sep, padding)
    s += ' '*padding
    s += bottom
    return s

def dict_to_string_std(fixed, rows, cols, padding = 0):
    """Returns a puzzle string of dimension 'boxsize' from a dictionary of 
    'fixed' cells with some suitably chosen row/column seperators."""
    ncols = len(cols)
    row_sep = '\n' + (2*ncols + 1)*'-' + '\n'
    col_sep = '|'
    return dict_to_string(fixed, rows, cols, padding, row_sep, col_sep, row_sep, row_sep)

def dict_to_string_xxx(fixed, nrows, ncols, padding = 0):
    rows = range(nrows)
    cols = range(ncols)
    return dict_to_string_std(fixed, rows, cols, padding)



#!/usr/bin/python

# Testing Hilton's claim on p.5 of:
#   "CPLS: Cropper's Question" by Bobga, Goldwasser, Hilton and Johnson.
#

# . . 7 3 5 . . 
# . . . . 6 1 5 
# . 6 . . . . 3
# 6 . . . . . .
# 4 . . . . . . 
# 2 . 1 . . . . 
# 3 4 2 . . . . 

from sys import argv

from ryser.examples import eg1, fail1
from ryser.hall import is_hall_on_interval

n = int(argv[1])
m = int(argv[2])

is_hall_on_interval(eg1, fail1[0], [6,7,11,15,18,19,20], n, m)




import networkx

def latin_graph(size):
    """The cartesian product of the complete graph with 'size' vertices with
    itself."""
    K = networkx.complete_graph(size)
    return networkx.cartesian_product(K, K)

def symmetric_latin_graph(size):
    """The graph described on p.20 of 
      "Completing Partial Latin Squares: Cropper's Question"
           by Bobga, Goldwasser, Hilton and Johnson. """
    G = latin_graph(size)
    def V(n):
        return [(i,j) for i in range(n) for j in range(n) if i <= j]
    G2 = G.subgraph(V(size))
    def suitable(v, w): 
        return v[0]<v[1] and v[1]==w[0] and w[0]<w[1]
    for v in G2.nodes():
        for w in G2.nodes():
            if suitable(v,w):
                G2.add_edge(v,w)
    return G2



from itertools import islice

from vizing.hall import hall_inequality_induced_by
from vizing.hall import hall_numbers as _hall_numbers_
from vizing.hall import hall_subgraph
from vizing.utils import powerset

from ryser.graphs import latin_graph, symmetric_latin_graph
from ryser.utils import list_assignment, vertex 

try:
    import matplotlib.pyplot as plt
except ImportError:
    pass

try:
    from networkx import draw_circular as draw
    from networkx import maximal_independent_set, write_graphml, write_dot
except ImportError:
    pass

def hall_inequality_on_cells_g(graph, lists, size, cells):
    """Decide if Hall's condition is satisfied for the subgraph of the latin
    graph induced by vertices in 'cells' and list assignment given by 'lists'."""
    vertices = [vertex(cell, size) for cell in cells]
    colours = range(1, size + 1)
    return hall_inequality_induced_by(graph, lists, colours, vertices)

def hall_inequality_on_cells(partial_latin_square, cells):
    """Decide whether Hall's condition is satisfied for cells."""
    size = partial_latin_square.size()
    fixed = partial_latin_square.fixed_cells()
    graph = latin_graph(size)
    lists = list_assignment(fixed, size)
    return hall_inequality_on_cells_g(graph, lists, size, cells)

def symmetric_hall_inequality_on_cells(partial_latin_square, cells):
    """Decide whether Hall's condition is satisfied for cells in symmetric
    case."""
    size = partial_latin_square.size()
    fixed = partial_latin_square.fixed_cells()
    graph = symmetric_latin_graph(size)
    lists = list_assignment(fixed, size)
    return hall_inequality_on_cells_g(graph, lists, size, cells)   

def hall_numbers(partial_latin_square, cells):
    """Returns a list of Hall numbers for the subgraph induced by cells in
    cells."""
    size = partial_latin_square.size()
    fixed = partial_latin_square.fixed_cells()
    cell_vertices = [vertex(x, size) for x in cells]
    H = latin_graph(size).subgraph(cell_vertices)
    L = list_assignment(fixed, size)
    colours = range(1, size + 1)
    return _hall_numbers_(H, L, colours)

def symmetric_hall_numbers(partial_latin_square, cells):
    """Returns a list of Hall numbers for the subgraph induced by cells in
    cells."""
    size = partial_latin_square.size()
    fixed = partial_latin_square.fixed_cells()
    cell_vertices = [vertex(x, size) for x in cells]
    H = symmetric_latin_graph(size).subgraph(cell_vertices)
    L = list_assignment(fixed, size)
    colours = range(1, size + 1)
    return _hall_numbers_(H, L, colours)

def hall_subgraphs(partial_latin_square, cells, with_labels = False, format = 'png'):
    """Draw symmetric Hall subgraphs."""
    size = partial_latin_square.size()
    fixed = partial_latin_square.fixed_cells()
    cell_vertices = [vertex(x, size) for x in cells]
    H = symmetric_latin_graph(size).subgraph(cell_vertices)
    L = list_assignment(fixed, size)
    for i in range(1, size + 1):
      Hi = hall_subgraph(H, L, i)
      try:
        I = maximal_independent_set(Hi)
        node_color = [v in I for v in Hi.nodes()]
      except:
        node_color = Hi.nodes()
      if Hi.number_of_nodes() > 0:
        if format == 'png':
          draw(Hi, node_color = node_color, with_labels = with_labels)
          plt.savefig("graph" + str(i) + ".png")
          plt.clf()
        elif format == 'graphml':
          write_graphml(Hi, "graph" + str(i) + ".graphml")
        elif format == 'dot':
          write_dot(Hi, "graph" + str(i) + ".dot")
        else:
          raise "No such format"

def is_hall_on_interval(pls, constant, exponent, n, m):
    for X in islice(powerset(exponent), n, m):
        Y = constant + list(X)
        result = hall_inequality_on_cells(pls, Y)
        print Y, result



# Created 18th December 2010. Last updated Thu Aug  9 16:14:34 BST 2012

class Latin:

  def __init__(self, P, size):
     self._P = P
     self._size = size

  def size(self):
     return self._size

  def fixed_cells(self):
     return self._P




# Matthew Henderson, 2012
# Created Wed Aug  8 15:37:51 BST 2012. Last updated Tue Aug 21 15:13:43 BST 2012

from distutils.core import setup

setup(
    name = 'ryser',
    version = '0.0.5',
    packages = ['ryser',],
    description = "Latin squares and related designs.",
    author = "Matthew Henderson",
    author_email = "matthew.james.henderson@gmail.com",
    scripts = ['bin/counterexample_investigation.py',
               'bin/hiltons_claim.py'],
    url = "http://packages.python.org/ryser/",
    download_url = "http://pypi.python.org/pypi/ryser/",
    keywords = [""],
    classifiers = [
        "Programming Language :: Python",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
    license = 'LICENSE.txt',
    long_description = open('README.txt').read(),
    install_requires=[
        "networkx >= 1.7.0",
        "vizing >= 0.0.11",
    ],
)




# Below, 'size' is the dimension of a latin square.
# Rows and columns are labelled: 0, 1, ... , size - 1
# Cells are labelled: 1, 2, ... , size^2

def cell(row, column, size):
    """The label of the cell in position (row, col)."""
    return (row * size) + column + 1

def row(cell, size): 
    """The row label of the row in a square of dimension 'size'
    containing the cell with label 'cell'."""
    return int((cell - 1)/size) 

def col(cell, size): 
    """The column label of the column in a square of dimension 'size'
    containing the cell with label 'cell'."""
    return (cell - 1) % size

def row_r(cell, size): 
    """A range of all labels of cells in the same row as 'cell'."""
    return range(row(cell, size)*size + 1, (row(cell, size) + 1)*size + 1)

def col_r(cell, size): 
    """A range of all labels of cells in the same column as 'cell'."""
    return range(col(cell, size) + 1, size**2 + 1, size)

def vertex(cell, size):
    """The row and column labels of 'cell', as a pair."""
    return (row(cell, size), col(cell, size))

def rect_r(tl, br, size):
    """A rectangular range which extends from tl in the top-left to br in the
    bottom right."""
    (i,j) = vertex(tl, size)
    (k,l) = vertex(br, size)
    cells = [(x, y) for x in range(i, k + 1) for y in range(j, l + 1)]
    return map(lambda x: cell(x[0], x[1], size), cells)

def list_assignment(partial_latin_square, size):
    """The (canonical) list assignment for a partial latin square. The list of
    a filled cell is the list containing just the element in that cell. The
    list of an empty cell contains only those symbols not already used in the
    same row and column as that cell."""
    P = partial_latin_square
    L = {}
    # initialise lists
    for i in range(1, size**2 + 1):
        if i in P.keys():
            L[row(i,size),col(i,size)] = [P[i]]
        else:
            L[row(i,size),col(i,size)] = range(1, size + 1)
    # update lists (remove used symbols from lists of same row/col)
    for i in range(1, size**2 + 1):
        if i in P.keys():
            # then remove P[i] from any list of a cell not in P from the same row/col
            for j in row_r(i, size) + col_r(i, size):
                if j not in P.keys():
                    if P[i] in L[row(j, size), col(j, size)]:
                        L[row(j, size), col(j, size)].remove(P[i])
    return L
